﻿using System;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Services.Authentication.DataAccess;
using DA = HeuristicLab.Services.Authentication.DataAccess;
using DT = HeuristicLab.Services.Authentication.DataTransfer;


namespace HeuristicLab.Services.Authentication
{

    public class ClientService : IClientService
    {

        public DataTransfer.Client GetClient(Guid id)
        {
            using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext())
            {
                DA.Client client = cmdc.Resources.OfType<DA.Client>().First(x => x.Id == id);
                return Convert.ToDto(client);
            }
        }

        public IEnumerable<DataTransfer.Client> GetClients()
        {
            using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext())
            {
                List<DT.Client> allClients = new List<DT.Client>();
                foreach (DA.Client dbClient in cmdc.Resources.OfType<DA.Client>())
                {
                    DT.Client client = Convert.ToDto(dbClient);
                    if (client != null)
                    {
                        DA.ResourceResourceGroup rrg = cmdc.ResourceResourceGroups.FirstOrDefault(x => x.ResourceId == client.Id);
                        if (rrg != null)
                            client.ResourceGroup = rrg.ResourceGroupId;
                        allClients.Add(client);
                    }
                }

                if (allClients.Count > 0)
                    return allClients;
                return null;
            }
        }

        public Guid AddClient(DataTransfer.Client dto)
        {
            using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext())
            {
                return InsertResource(cmdc, dto);
            }
        }


        public bool UpdateClient(DataTransfer.Client dto)
        {
            throw new NotImplementedException();
        }

        public bool DeleteClient(DataTransfer.Client dto)
        {
            using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext())
            {
                return RemoveResource(cmdc, dto);
            }
        }

        public DataTransfer.ResourceGroup GetResourceGroup(Guid id)
        {
            using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext())
            {
                DA.ResourceGroup client = cmdc.Resources.OfType<DA.ResourceGroup>().First(x => x.Id == id);
                return Convert.ToDto(client);
            }
        }

        public IEnumerable<DataTransfer.ResourceGroup> GetResourceGroups()
        {
            using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext())
            {
                List<DT.ResourceGroup> allClients = new List<DT.ResourceGroup>();
                foreach (DA.ResourceGroup dbResourceGroup in cmdc.Resources.OfType<DA.ResourceGroup>())
                {
                    DT.ResourceGroup resourceGroup = Convert.ToDto(dbResourceGroup);
                    if (resourceGroup != null)
                        allClients.Add(resourceGroup);
                }

                if (allClients.Count > 0)
                    return allClients;
                return null;
            }
        }

        public Guid AddResourceGroup(DataTransfer.ResourceGroup dto)
        {
            using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext())
            {
                return InsertResource(cmdc, dto);
            }
        }

        public bool UpdateResourceGroup(DataTransfer.ResourceGroup dto)
        {
            throw new NotImplementedException();
        }

        public bool DeleteResourceGroup(DataTransfer.ResourceGroup dto)
        {
            using (ClientManagmentDataContext cmdc = new ClientManagmentDataContext())
            {
                return RemoveResource(cmdc, dto);
            }
        }

        public IEnumerable<DataTransfer.Resource> GetMembers(Guid id)
        {
            throw new NotImplementedException();
        }

        public IEnumerable<DataTransfer.Resource> GetRootResources()
        {
            throw new NotImplementedException();
        }

        private IEnumerable<DataTransfer.Resource> GetMembers(ClientManagmentDataContext cmdc, Guid id)
        {
            // TODO IMPLEMENTATION
            return null;
        }

        private Guid InsertResource(ClientManagmentDataContext cmdc, DT.Resource dto)
        {
            ResourceResourceGroup rrg = null;
            if (dto.ResourceGroup != Guid.Empty)
            {
                ResourceGroup grp = cmdc.Resources.OfType<DA.ResourceGroup>().FirstOrDefault(x => x.Id == dto.ResourceGroup);
                if (grp == null)
                    return Guid.Empty;
                rrg = new ResourceResourceGroup();
                rrg.ResourceGroup = grp;
            }
            DA.Resource entity = Convert.ToEntity(dto);
            if (entity != null)
            {
                entity.Id = Guid.NewGuid();
                if (rrg != null)
                    rrg.Client = entity;
                cmdc.Resources.InsertOnSubmit(entity);
                if (rrg != null)
                    cmdc.ResourceResourceGroups.InsertOnSubmit(rrg);
                cmdc.SubmitChanges();
                return entity.Id;
            }
            return Guid.Empty;
        }

        private static bool RemoveResource(ClientManagmentDataContext cmdc, DT.Resource dto)
        {
            RemoveRRGEntry(cmdc, dto.Id);
            DA.Resource entity = cmdc.Resources.FirstOrDefault(x => x.Id == dto.Id);
            if (entity != null)
            {
                cmdc.Resources.DeleteOnSubmit(entity);
                cmdc.SubmitChanges();
                return true;
            }
            return false;
        }

        private static void RemoveRRGEntry(ClientManagmentDataContext cmdc, Guid resID)
        {
            foreach (var rrg in cmdc.ResourceResourceGroups.Where(x => x.ResourceId == resID))
                cmdc.ResourceResourceGroups.DeleteOnSubmit(rrg);
            foreach (var rrg in cmdc.ResourceResourceGroups.Where(x => x.ResourceGroupId == resID))
            {
                // Begin Cascade Delete
                RemoveRRGEntry(cmdc, rrg.Client.Id);
                cmdc.Resources.DeleteOnSubmit(rrg.Client);
                // End Cascade Delete
                cmdc.ResourceResourceGroups.DeleteOnSubmit(rrg);
            }
        }
    }
}